from abc import abstractmethod

from bmipy import Bmi


class Xmi(Bmi):
    """
    This class extends the CSDMS Basic Model Interface

    The extension to the BMI is twofold:

    - the model's outer convergence loop is exposed to facilitate coupling at
      this level

    - a model can have sub-components which share the time stepping but have
      their own convergence loop

    It does not change anything in the BMI interface, so models implementing
    the XMI interface are compatible with BMI

    """

    @abstractmethod
    def prepare_time_step(self, dt: float) -> None:
        """ """
        ...

    @abstractmethod
    def do_time_step(self) -> None:
        """ """
        ...

    @abstractmethod
    def finalize_time_step(self) -> None:
        """ """
        ...

    @abstractmethod
    def get_subcomponent_count(self) -> int:
        """ """
        ...

    @abstractmethod
    def prepare_solve(self, component_id: int) -> None:
        """ """
        ...

    @abstractmethod
    def solve(self, component_id: int) -> bool:
        """ """
        ...

    @abstractmethod
    def finalize_solve(self, component_id: int) -> None:
        """ """
        ...



import codecs
import os.path
import sys

from setuptools import find_namespace_packages, setup

# edit author dictionary as necessary
author_dict = {
    "Martijn Russcher": "Martijn.Russcher@deltares.nl",
    "Julian Hofer": "Julian.Hofer@deltares.nl",
    "Joseph D. Hughes": "jdhughes@usgs.gov",
}
__author__ = ", ".join(author_dict.keys())
__author_email__ = ", ".join(s for _, s in author_dict.items())


def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), "r") as fp:
        return fp.read()


def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith("__version__"):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")


long_description = read("README.md")


setup(
    name="xmipy",
    description="xmipy is an extension to the bmipy Python package.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=__author__,
    author_email=__author_email__,
    url="https://github.com/Deltares/xmipy.git",
    license="CC0",
    platforms="Windows, Mac OS-X, Linux",
    install_requires=["bmipy", "numpy"],
    extras_require={
        "tests": ["pytest", "pytest-cov", "requests", "mfpymake", "flopy"],
        "lint": [
            "mypy",
            "black",
            "isort",
        ],
    },
    python_requires=">=3.7",
    packages=find_namespace_packages(exclude=("tests", "examples")),
    package_data={"xmipy": ["py.typed"]},
    version=get_version("xmipy/__init__.py"),
    classifiers=["Topic :: Scientific/Engineering :: Hydrology"],
    zip_safe=False,
)
